package de.jottyfan.timetrack.modules.done.fullcalendar;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * 
 * @author henkej
 *
 */
public abstract class FCEventBean implements Serializable {
	private static final long serialVersionUID = 1L;

	private String title;
	private Boolean allDay;
	private Date start;
	private Date end;
	private Boolean editable;
	private Boolean startEditable;
	private Boolean durationEditable;
	private String color;
	private String url;

	public FCEventBean(String title, Date start) {
		super();
		this.title = title;
		this.start = start;
	}

	protected String toJavascriptDate(Date date){
		if (date == null) {
			return null;
		}
		StringBuilder buf = new StringBuilder("new Date(");
		Calendar cal = new GregorianCalendar();
		cal.setTime(date);
		buf.append(cal.get(Calendar.YEAR)).append(",");
		buf.append(cal.get(Calendar.MONTH)).append(",");
		buf.append(cal.get(Calendar.DAY_OF_MONTH)).append(",");
		buf.append(cal.get(Calendar.HOUR_OF_DAY)).append(",");
		buf.append(cal.get(Calendar.MINUTE)).append(",");
		buf.append(cal.get(Calendar.SECOND));
		buf.append(")");
		return buf.toString();
	}
	
	/**
	 * get extended fields to json string
	 */
	public abstract String getExtendedFields();
	
	/**
	 * adds a param to buf if value is not null
	 * 
	 * @param buf the buffer
	 * @param name the name
	 * @param value the value
	 * @param encapsulate if true, encapsulate value by '
	 * @param appendComma if true, append a comma
	 */
	public void addParam(StringBuilder buf, String name, String value, boolean encapsulate, boolean appendComma) {
		if (value != null) {
			buf.append(name).append(":");
			buf.append(encapsulate ? "'" : "").append(value).append(encapsulate ? "'" : "");
			buf.append(appendComma ? "," : "");
		}
	}
	
	/**
	 * adds a param to buf if value is not null and append a comma
	 * 
	 * @param buf the buffer
	 * @param name the name
	 * @param value the value
	 * @param encapsulate if true, encapsulate value by '
	 */
	public void addParam(StringBuilder buf, String name, String value, boolean encapsulate) {
		addParam(buf, name, value, encapsulate, true);
	}
	
	/**
	 * adds a param to buf if value is not null and encapsulate it with ' and add a comma
	 * 
	 * @param buf the buffer
	 * @param name the name
	 * @param value the value
	 */
	public void addParam(StringBuilder buf, String name, String value) {
		addParam(buf, name, value, true, true);
	}
	
	/**
	 * adds a param to buf if value is not null and do not encapsulate it with ' but add a comma
	 * 
	 * @param buf the buffer
	 * @param name the name
	 * @param value the value
	 */
	public void addParam(StringBuilder buf, String name, Boolean value) {
		addParam(buf, name, value == null ? null : value.toString(), false, true);
	}
	
	public String toJson() {
		StringBuilder buf = new StringBuilder("{");
		addParam(buf, "title", title);
		addParam(buf, "allDay", allDay);
		addParam(buf, "editable", editable);
		addParam(buf, "startEditable", startEditable);
		addParam(buf, "durationEditable", durationEditable);
		addParam(buf, "color", color);
		addParam(buf, "end", toJavascriptDate(end), false);
		addParam(buf, "url", url);

		buf.append(getExtendedFields());
		// this is the last element, we need no trailing ,
		addParam(buf, "start", toJavascriptDate(start), false, false);
		buf.append("}");
		return buf.toString();
	}

	public String getUrl() {
		return url;
	}
	
	public void setUrl(String url) {
		this.url = url;
	}
	
	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public Boolean getAllDay() {
		return allDay;
	}

	public void setAllDay(Boolean allDay) {
		this.allDay = allDay;
	}

	public Date getStart() {
		return start;
	}

	public void setStart(Date start) {
		this.start = start;
	}

	public Date getEnd() {
		return end;
	}

	public void setEnd(Date end) {
		this.end = end;
	}

	public Boolean getEditable() {
		return editable;
	}

	public void setEditable(Boolean editable) {
		this.editable = editable;
	}

	public Boolean getDurationEditable() {
		return durationEditable;
	}

	public void setDurationEditable(Boolean durationEditable) {
		this.durationEditable = durationEditable;
	}

	public String getColor() {
		return color;
	}

	public void setColor(String color) {
		this.color = color;
	}

	public static long getSerialversionuid() {
		return serialVersionUID;
	}

	public Boolean getStartEditable() {
		return startEditable;
	}

	public void setStartEditable(Boolean startEditable) {
		this.startEditable = startEditable;
	}
}
